@@ -18,7 +18,7 @@
 
 #ifdef MONGOC_ENABLE_SSL_OPENSSL
 
-#include <bson.h>
+#include <bson/bson.h>
 #include <limits.h>
 #include <openssl/bio.h>
 #include <openssl/ssl.h>
@@ -41,7 +41,7 @@
 #endif
 
 #if OPENSSL_VERSION_NUMBER < 0x10100000L
-static mongoc_mutex_t *gMongocOpenSslThreadLocks;
+static bson_mutex_t *gMongocOpenSslThreadLocks;
 
 static void
 _mongoc_openssl_thread_startup (void);
@@ -493,6 +493,11 @@ _mongoc_openssl_ctx_new (mongoc_ssl_opt_t *opt)
    ssl_ctx_options |= SSL_OP_NO_COMPRESSION;
 #endif
 
+/* man SSL_get_options says: "SSL_OP_NO_RENEGOTIATION options were added in OpenSSL 1.1.1" */
+#if OPENSSL_VERSION_NUMBER >= 0x10101000L
+   ssl_ctx_options |= SSL_OP_NO_RENEGOTIATION;
+#endif
+
    SSL_CTX_set_options (ctx, ssl_ctx_options);
 
 /* only defined in special build, using:
@@ -626,9 +631,9 @@ _mongoc_openssl_thread_locking_callback (int mode,
                                          int line)
 {
    if (mode & CRYPTO_LOCK) {
-      mongoc_mutex_lock (&gMongocOpenSslThreadLocks[type]);
+      bson_mutex_lock (&gMongocOpenSslThreadLocks[type]);
    } else {
-      mongoc_mutex_unlock (&gMongocOpenSslThreadLocks[type]);
+      bson_mutex_unlock (&gMongocOpenSslThreadLocks[type]);
    }
 }
 
@@ -637,11 +642,11 @@ _mongoc_openssl_thread_startup (void)
 {
    int i;
 
-   gMongocOpenSslThreadLocks = (mongoc_mutex_t *) OPENSSL_malloc (
-      CRYPTO_num_locks () * sizeof (mongoc_mutex_t));
+   gMongocOpenSslThreadLocks = (bson_mutex_t *) OPENSSL_malloc (
+      CRYPTO_num_locks () * sizeof (bson_mutex_t));
 
    for (i = 0; i < CRYPTO_num_locks (); i++) {
-      mongoc_mutex_init (&gMongocOpenSslThreadLocks[i]);
+      bson_mutex_init (&gMongocOpenSslThreadLocks[i]);
    }
 
    if (!CRYPTO_get_locking_callback ()) {
@@ -665,7 +670,7 @@ _mongoc_openssl_thread_cleanup (void)
    }
 
    for (i = 0; i < CRYPTO_num_locks (); i++) {
-      mongoc_mutex_destroy (&gMongocOpenSslThreadLocks[i]);
+      bson_mutex_destroy (&gMongocOpenSslThreadLocks[i]);
    }
    OPENSSL_free (gMongocOpenSslThreadLocks);
 }
