@@ -15,27 +15,26 @@
  */
 
 #include <bson/bson.h>
-#include <bson/bson-string.h>
 
-#include "mongoc/mongoc-config.h"
-#include "mongoc/mongoc-error.h"
-#include "mongoc/mongoc-trace-private.h"
-#include "mongoc/mongoc-topology-scanner-private.h"
-#include "mongoc/mongoc-stream-private.h"
-#include "mongoc/mongoc-stream-socket.h"
+#include "mongoc-config.h"
+#include "mongoc-error.h"
+#include "mongoc-trace-private.h"
+#include "mongoc-topology-scanner-private.h"
+#include "mongoc-stream-private.h"
+#include "mongoc-stream-socket.h"
 
-#include "mongoc/mongoc-handshake.h"
-#include "mongoc/mongoc-handshake-private.h"
+#include "mongoc-handshake.h"
+#include "mongoc-handshake-private.h"
 
 #ifdef MONGOC_ENABLE_SSL
-#include "mongoc/mongoc-stream-tls.h"
+#include "mongoc-stream-tls.h"
 #endif
 
-#include "mongoc/mongoc-counters-private.h"
-#include "mongoc/utlist.h"
-#include "mongoc/mongoc-topology-private.h"
-#include "mongoc/mongoc-host-list-private.h"
-#include "mongoc/mongoc-uri-private.h"
+#include "mongoc-counters-private.h"
+#include "utlist.h"
+#include "mongoc-topology-private.h"
+#include "mongoc-host-list-private.h"
+#include "mongoc-uri-private.h"
 
 #undef MONGOC_LOG_DOMAIN
 #define MONGOC_LOG_DOMAIN "topology_scanner"
@@ -329,6 +328,20 @@ mongoc_topology_scanner_scan (mongoc_topology_scanner_t *ts, uint32_t id)
    /* if setup fails the node stays in the scanner. destroyed after the scan. */
 }
 
+void
+mongoc_topology_scanner_disconnect (mongoc_topology_scanner_t *scanner)
+{
+   mongoc_topology_scanner_node_t *node;
+
+   BSON_ASSERT (scanner);
+   node = scanner->nodes;
+
+   while (node) {
+      mongoc_topology_scanner_node_disconnect (node, false);
+      node = node->next;
+   }
+}
+
 void
 mongoc_topology_scanner_node_retire (mongoc_topology_scanner_node_t *node)
 {
@@ -847,7 +860,7 @@ bool
 mongoc_topology_scanner_node_in_cooldown (mongoc_topology_scanner_node_t *node,
                                           int64_t when)
 {
-   if (node->last_failed == -1) {
+   if (node->last_failed == -1 || node->ts->bypass_cooldown) {
       return false; /* node is new, or connected */
    }
 
@@ -871,6 +884,9 @@ mongoc_topology_scanner_in_cooldown (mongoc_topology_scanner_t *ts,
 {
    mongoc_topology_scanner_node_t *node;
 
+   if (ts->bypass_cooldown) {
+      return false;
+   }
    DL_FOREACH (ts->nodes, node)
    {
       if (!mongoc_topology_scanner_node_in_cooldown (node, when)) {
