@@ -17,6 +17,7 @@
 
 #include "bson/bson.h"
 
+#include "mongoc/mongoc-change-stream-private.h"
 #include "mongoc/mongoc-collection-private.h"
 #include "mongoc/mongoc-config.h"
 #include "mongoc/mongoc-cursor-private.h"
@@ -24,9 +25,8 @@
 #include "mongoc/mongoc-server-description-private.h"
 #include "mongoc/mongoc-topology-description-private.h"
 #include "mongoc/mongoc-topology-private.h"
-#include "mongoc/mongoc-util-private.h"
-#include "mongoc/mongoc-util-private.h"
 #include "mongoc/mongoc-uri-private.h"
+#include "mongoc/mongoc-util-private.h"
 
 #include "json-test-operations.h"
 #include "json-test.h"
@@ -64,6 +64,8 @@ json_test_ctx_init (json_test_ctx_t *ctx,
    int i;
    bson_error_t error;
 
+   memset (ctx, 0, sizeof (*ctx));
+
    ctx->client = client;
    ctx->db = db;
    ctx->collection = collection;
@@ -82,7 +84,7 @@ json_test_ctx_init (json_test_ctx_t *ctx,
    ctx->has_sessions = test_framework_session_timeout_minutes () > -1 &&
                        test_framework_skip_if_no_crypto ();
 
-   /* transactions tests require two sessions named session1 and session2,
+   /* transactions tests require two sessions named session0 and session1,
     * retryable writes use one explicit session or none */
    if (ctx->has_sessions) {
       for (i = 0; i < 2; i++) {
@@ -132,6 +134,8 @@ json_test_ctx_cleanup (json_test_ctx_t *ctx)
    bson_destroy (&ctx->lsids[1]);
    bson_destroy (&ctx->events);
    mongoc_uri_destroy (ctx->test_framework_uri);
+   bson_destroy (ctx->sent_lsids[0]);
+   bson_destroy (ctx->sent_lsids[1]);
 }
 
 
@@ -596,6 +600,13 @@ add_request_to_bulk (mongoc_bulk_operation_t *bulk,
       BSON_APPEND_DOCUMENT (&opts, "collation", &collation);
    }
 
+   if (bson_has_field (&args, "hint")) {
+      bson_value_t hint;
+      bson_lookup_value (&args, "hint", &hint);
+      BSON_APPEND_VALUE (&opts, "hint", &hint);
+      bson_value_destroy (&hint);
+   }
+
    if (!strcmp (name, "deleteMany")) {
       bson_t filter;
 
@@ -1759,6 +1770,13 @@ gridfs_download (mongoc_database_t *db,
    return true;
 }
 
+static bool
+op_error (const bson_t *operation)
+{
+   return bson_has_field (operation, "error") &&
+          bson_lookup_bool (operation, "error");
+}
+
 bool
 json_test_operation (json_test_ctx_t *ctx,
                      const bson_t *test,
@@ -1838,16 +1856,23 @@ json_test_operation (json_test_ctx_t *ctx,
          res = find_one (c, test, operation, session, read_prefs, reply);
       } else if (!strcmp (op_name, "aggregate")) {
          res = aggregate (c, test, operation, session, read_prefs, reply);
-      } else if (!strcmp (op_name, "listIndexes")) {
+      } else if (!strcmp (op_name, "listIndexes") ||
+                 !strcmp (op_name, "listIndexNames")) {
          res = list_indexes (c, test, operation, session, reply);
       } else if (!strcmp (op_name, "watch")) {
          bson_t pipeline = BSON_INITIALIZER;
          mongoc_change_stream_destroy (ctx->change_stream);
          ctx->change_stream = mongoc_collection_watch (c, &pipeline, NULL);
+         res = (op_error (operation) == (0 != ctx->change_stream->err.code));
+         if (!res) {
+            test_error ("expected error=%s, but actual error='%s'",
+                        op_error (operation) ? "true" : "false",
+                        ctx->change_stream->err.message);
+         }
+
          bson_init (reply);
          bson_destroy (&pipeline);
-      } else if (!strcmp (op_name, "mapReduce") ||
-                 !strcmp (op_name, "listIndexNames")) {
+      } else if (!strcmp (op_name, "mapReduce")) {
          test_error ("operation not implemented in libmongoc");
       } else {
          test_error ("unrecognized collection operation name %s", op_name);
@@ -1865,6 +1890,13 @@ json_test_operation (json_test_ctx_t *ctx,
          bson_t pipeline = BSON_INITIALIZER;
          mongoc_change_stream_destroy (ctx->change_stream);
          ctx->change_stream = mongoc_database_watch (db, &pipeline, NULL);
+         res = (op_error (operation) == (0 != ctx->change_stream->err.code));
+         if (!res) {
+            test_error ("expected error=%s, but actual error='%s'",
+                        op_error (operation) ? "true" : "false",
+                        ctx->change_stream->err.message);
+         }
+
          bson_init (reply);
          bson_destroy (&pipeline);
       } else if (!strcmp (op_name, "listCollectionObjects")) {
@@ -1882,6 +1914,17 @@ json_test_operation (json_test_ctx_t *ctx,
          res = commit_transaction (named_session, test, operation, reply);
       } else if (!strcmp (op_name, "abortTransaction")) {
          res = abort_transaction (named_session, test, operation, reply);
+      } else if (!strcmp (op_name, "endSession")) {
+         mongoc_client_session_destroy (named_session);
+         if (0 == strcmp (obj_name, "session0")) {
+            ctx->sessions[0] = NULL;
+         } else if (0 == strcmp (obj_name, "session1")) {
+            ctx->sessions[1] = NULL;
+         } else {
+            test_error ("unrecognized session: %s", op_name);
+         }
+         res = true;
+         bson_init (reply);
       } else {
          test_error ("unrecognized session operation name %s", op_name);
       }
@@ -1900,6 +1943,22 @@ json_test_operation (json_test_ctx_t *ctx,
             client, session->server_id, operation, "arguments.failPoint");
 
          mongoc_client_destroy (client);
+      } else if (!strcmp (op_name, "assertSessionNotDirty")) {
+         BSON_ASSERT (!session->server_session->dirty);
+      } else if (!strcmp (op_name, "assertSessionDirty")) {
+         BSON_ASSERT (session->server_session->dirty);
+      } else if (!strcmp (op_name, "assertSameLsidOnLastTwoCommands")) {
+         if (!ctx->sent_lsids[0] || !ctx->sent_lsids[1]) {
+            test_error ("attempting to check last two session IDs, but test "
+                        "runner has not captured them");
+         }
+         BSON_ASSERT (bson_equal (ctx->sent_lsids[0], ctx->sent_lsids[1]));
+      } else if (!strcmp (op_name, "assertDifferentLsidOnLastTwoCommands")) {
+         if (!ctx->sent_lsids[0] || !ctx->sent_lsids[1]) {
+            test_error ("attempting to check last two session IDs, but test "
+                        "runner has not captured them");
+         }
+         BSON_ASSERT (!bson_equal (ctx->sent_lsids[0], ctx->sent_lsids[1]));
       } else {
          test_error ("unrecognized session operation name %s", op_name);
       }
@@ -1912,6 +1971,13 @@ json_test_operation (json_test_ctx_t *ctx,
          bson_t pipeline = BSON_INITIALIZER;
          mongoc_change_stream_destroy (ctx->change_stream);
          ctx->change_stream = mongoc_client_watch (c->client, &pipeline, NULL);
+         res = (op_error (operation) == (0 != ctx->change_stream->err.code));
+         if (!res) {
+            test_error ("expected error=%s, but actual error='%s'",
+                        op_error (operation) ? "true" : "false",
+                        ctx->change_stream->err.message);
+         }
+
          bson_init (reply);
          bson_destroy (&pipeline);
       } else if (!strcmp (op_name, "listDatabaseObjects")) {
