@@ -593,6 +593,9 @@ get_exists_operator (const bson_value_t *value, bool *exists);
 static bool
 get_empty_operator (const bson_value_t *value, bool *exists);
 
+static bool
+get_type_operator (const bson_value_t *value, bson_type_t *out);
+
 static bool
 is_empty_doc_or_array (const bson_value_t *value);
 
@@ -833,8 +836,10 @@ derive (match_ctx_t *ctx, match_ctx_t *derived, const char *key)
  *       if its key-value pairs are a simple superset of pattern's. Order
  *       matters.
  *
- *       The only special pattern syntaxes are "field": {"$exists": true/false}
- *       and "field": {"$empty": true/false}.
+ *       The only special pattern syntaxes are:
+ *         "field": {"$exists": true/false}
+ *         "field": {"$empty": true/false}
+ *         "field": {"$$type": "type string"}
  *
  *       The first key matches case-insensitively if ctx->is_command.
  *
@@ -861,8 +866,10 @@ match_bson_with_ctx (const bson_t *doc, const bson_t *pattern, match_ctx_t *ctx)
    bool is_first = true;
    bool is_exists_operator;
    bool is_empty_operator;
+   bool is_type_operator;
    bool exists;
    bool empty;
+   bson_type_t bson_type;
    bool found;
    bson_iter_t doc_iter;
    bson_value_t doc_value;
@@ -895,6 +902,9 @@ match_bson_with_ctx (const bson_t *doc, const bson_t *pattern, match_ctx_t *ctx)
       /* is value {"$empty": true} or {"$empty": false} ? */
       is_empty_operator = get_empty_operator (value, &empty);
 
+      /* is value {"$$type": "string" } ? */
+      is_type_operator = get_type_operator (value, &bson_type);
+
       derive (ctx, &derived, key);
 
       if (ctx->visitor_fn) {
@@ -926,6 +936,11 @@ match_bson_with_ctx (const bson_t *doc, const bson_t *pattern, match_ctx_t *ctx)
             match_err (&derived, "%s found", empty ? "" : " not");
             goto fail;
          }
+      } else if (is_type_operator) {
+         if (doc_value.value_type != bson_type) {
+            match_err (&derived, "incorrect type");
+            goto fail;
+         }
       } else if (!match_bson_value (&doc_value, value, &derived)) {
          goto fail;
       }
@@ -1083,6 +1098,88 @@ get_empty_operator (const bson_value_t *value, bool *empty)
 }
 
 
+/*--------------------------------------------------------------------------
+ *
+ * get_type_operator --
+ *
+ *       Is value a subdocument like {"$$type": "BSON type string"}?
+ *
+ * Returns:
+ *       True if the value is a subdocument with the first key "$$type",
+ *       and sets the @bson_type.
+ *
+ * Side effects:
+ *       If the function returns true, *@bson_type is set.
+ *
+ *--------------------------------------------------------------------------
+ */
+
+static bool
+get_type_operator (const bson_value_t *value, bson_type_t *out)
+{
+   bson_t bson;
+   bson_iter_t iter;
+   const char *value_string;
+
+   /* See list of aliases on this page:
+    * https://docs.mongodb.com/manual/reference/bson-types/ */
+   if (value->value_type == BSON_TYPE_DOCUMENT &&
+       bson_init_from_value (&bson, value) &&
+       bson_iter_init_find (&iter, &bson, "$$type")) {
+      value_string = bson_iter_utf8 (&iter, NULL);
+      if (0 == strcasecmp ("double", value_string)) {
+         *out = BSON_TYPE_DOUBLE;
+      } else if (0 == strcasecmp ("string", value_string)) {
+         *out = BSON_TYPE_UTF8;
+      } else if (0 == strcasecmp ("object", value_string)) {
+         *out = BSON_TYPE_DOCUMENT;
+      } else if (0 == strcasecmp ("array", value_string)) {
+         *out = BSON_TYPE_ARRAY;
+      } else if (0 == strcasecmp ("binData", value_string)) {
+         *out = BSON_TYPE_BINARY;
+      } else if (0 == strcasecmp ("undefined", value_string)) {
+         *out = BSON_TYPE_UNDEFINED;
+      } else if (0 == strcasecmp ("objectId", value_string)) {
+         *out = BSON_TYPE_OID;
+      } else if (0 == strcasecmp ("bool", value_string)) {
+         *out = BSON_TYPE_BOOL;
+      } else if (0 == strcasecmp ("date", value_string)) {
+         *out = BSON_TYPE_DATE_TIME;
+      } else if (0 == strcasecmp ("null", value_string)) {
+         *out = BSON_TYPE_NULL;
+      } else if (0 == strcasecmp ("regex", value_string)) {
+         *out = BSON_TYPE_REGEX;
+      } else if (0 == strcasecmp ("dbPointer", value_string)) {
+         *out = BSON_TYPE_DBPOINTER;
+      } else if (0 == strcasecmp ("javascript", value_string)) {
+         *out = BSON_TYPE_CODE;
+      } else if (0 == strcasecmp ("symbol", value_string)) {
+         *out = BSON_TYPE_SYMBOL;
+      } else if (0 == strcasecmp ("javascriptWithScope", value_string)) {
+         *out = BSON_TYPE_CODEWSCOPE;
+      } else if (0 == strcasecmp ("int", value_string)) {
+         *out = BSON_TYPE_INT32;
+      } else if (0 == strcasecmp ("timestamp", value_string)) {
+         *out = BSON_TYPE_TIMESTAMP;
+      } else if (0 == strcasecmp ("long", value_string)) {
+         *out = BSON_TYPE_INT64;
+      } else if (0 == strcasecmp ("decimal", value_string)) {
+         *out = BSON_TYPE_DECIMAL128;
+      } else if (0 == strcasecmp ("minKey", value_string)) {
+         *out = BSON_TYPE_MINKEY;
+      } else if (0 == strcasecmp ("maxKey", value_string)) {
+         *out = BSON_TYPE_MAXKEY;
+      } else {
+         fprintf (stderr, "unrecognized $$type value: %s\n", value_string);
+         abort ();
+      }
+      return true;
+   }
+
+   return false;
+}
+
+
 /*--------------------------------------------------------------------------
  *
  * is_empty_doc_or_array --
@@ -1394,11 +1491,21 @@ match_bson_value (const bson_value_t *doc,
       return true;
 
    case BSON_TYPE_DBPOINTER:
-      test_error ("DBPointer comparison not implemented");
-      abort ();
+      ret = (0 == strcmp (doc->value.v_dbpointer.collection,
+                          pattern->value.v_dbpointer.collection) &&
+             bson_oid_equal (&doc->value.v_dbpointer.oid,
+                             &pattern->value.v_dbpointer.oid));
+      break;
+
    case BSON_TYPE_DECIMAL128:
-      test_error ("Decimal128 comparison not implemented");
-      abort ();
+      ret = (doc->value.v_decimal128.low == pattern->value.v_decimal128.low &&
+             doc->value.v_decimal128.high == pattern->value.v_decimal128.high);
+      if (!ret) {
+         match_err (ctx,
+                    "Decimal128 is not an exact binary match (though "
+                    "numeric values may be equal)");
+      }
+      break;
    default:
       test_error ("unexpected value type %d: %s",
                   doc->value_type,
