@@ -78,6 +78,8 @@ test_writer_install (TestSuite *suite);
 
 /* libmongoc */
 
+extern void
+test_aggregate_install (TestSuite *suite);
 extern void
 test_array_install (TestSuite *suite);
 extern void
@@ -133,6 +135,8 @@ test_log_install (TestSuite *suite);
 extern void
 test_matcher_install (TestSuite *suite);
 extern void
+test_mongos_pinning_install (TestSuite *suite);
+extern void
 test_handshake_install (TestSuite *suite);
 extern void
 test_queue_install (TestSuite *suite);
@@ -218,6 +222,8 @@ extern void
 test_crud_install (TestSuite *suite);
 extern void
 test_apm_install (TestSuite *suite);
+extern void
+test_client_side_encryption_install (TestSuite *suite);
 
 typedef struct {
    mongoc_log_level_t level;
@@ -2284,6 +2290,31 @@ test_framework_skip_if_no_failpoint (void)
    return 1;
 }
 
+int
+test_framework_skip_if_no_client_side_encryption (void)
+{
+   char *aws_secret_access_key;
+   char *aws_access_key_id;
+   bool has_creds;
+
+   aws_secret_access_key =
+      test_framework_getenv ("MONGOC_TEST_AWS_SECRET_ACCESS_KEY");
+   aws_access_key_id = test_framework_getenv ("MONGOC_TEST_AWS_ACCESS_KEY_ID");
+
+   has_creds = (NULL != aws_secret_access_key) && (NULL != aws_access_key_id);
+
+   bson_free (aws_secret_access_key);
+   bson_free (aws_access_key_id);
+
+   if (has_creds) {
+#ifdef MONGOC_ENABLE_CLIENT_SIDE_ENCRYPTION
+      return 1; /* 1 == proceed. */
+#else
+      return 0; /* 0 == do not proceed. */
+#endif
+   }
+   return 0; /* 0 == do not proceed. */
+}
 
 void
 test_framework_resolve_path (const char *path, char *resolved)
@@ -2347,6 +2378,7 @@ main (int argc, char *argv[])
 
    /* libmongoc */
 
+   test_aggregate_install (&suite);
    test_array_install (&suite);
    test_async_install (&suite);
    test_buffer_install (&suite);
@@ -2375,6 +2407,7 @@ main (int argc, char *argv[])
    test_list_install (&suite);
    test_log_install (&suite);
    test_matcher_install (&suite);
+   test_mongos_pinning_install (&suite);
    test_queue_install (&suite);
    test_primary_stepdown_install (&suite);
    test_read_concern_install (&suite);
@@ -2419,6 +2452,7 @@ main (int argc, char *argv[])
    test_counters_install (&suite);
    test_crud_install (&suite);
    test_apm_install (&suite);
+   test_client_side_encryption_install (&suite);
 
    ret = TestSuite_Run (&suite);
 
