@@ -4,12 +4,20 @@
 #include "json-test-operations.h"
 #include "test-libmongoc.h"
 
-static void
+static bool
 crud_test_operation_cb (json_test_ctx_t *ctx,
                         const bson_t *test,
                         const bson_t *operation)
 {
-   json_test_operation (ctx, test, operation, ctx->collection, NULL);
+   bson_t reply;
+   bool res;
+
+   res =
+      json_test_operation (ctx, test, operation, ctx->collection, NULL, &reply);
+
+   bson_destroy (&reply);
+
+   return res;
 }
 
 static void
@@ -17,6 +25,7 @@ test_crud_cb (bson_t *scenario)
 {
    json_test_config_t config = JSON_TEST_CONFIG_INIT;
    config.run_operation_cb = crud_test_operation_cb;
+   config.command_started_events_only = true;
    config.scenario = scenario;
    run_json_general_test (&config);
 }
@@ -27,11 +36,15 @@ test_all_spec_tests (TestSuite *suite)
    char resolved[PATH_MAX];
 
    test_framework_resolve_path (JSON_DIR "/crud", resolved);
-   install_json_test_suite (suite, resolved, &test_crud_cb);
+
+   install_json_test_suite_with_check (suite,
+                                       resolved,
+                                       &test_crud_cb,
+                                       test_framework_skip_if_no_sessions);
 }
 
 void
 test_crud_install (TestSuite *suite)
 {
    test_all_spec_tests (suite);
-}
\ No newline at end of file
+}
